#! /usr/bin/python
# -*- coding: utf-8 -*-

################################################################################
##@package Highlighter.py
#
# Paquete que implementa el resaltador de texto para los archivos *.ly de
# Lylipond. Basada en los ejemplos de PyQt 4.6>
#
# Para el proyecto Toccata, IDE para desarrollo de partituras con Lilypond
# en su versión 2010.
#
# @author Javier Salazar Loyola <jsalazar@elo.utfsm.cl>
################################################################################

from PyQt4 import QtGui, QtCore

## Clase encargada de todos los procesos de resaltado del documento que
#  tiene como padre (argumento parent del constructor).
#  Debido a la implementación de su clase padre, su función principal,
#  highlightBlock() será llamada cada vez que sea necesario, por lo que sólo
#  es necesario asociar el texto con la instancia de esta clase.
class LilypondHighlighter(QtGui.QSyntaxHighlighter):
    ## Constructor del objeto, inicializa, principalmente, las variables
    # para el resaltado de la sintaxis
    # @param parent Objeto al que se asocia el resaltado de sintaxis.
    def __init__(self, parent = None):
        super(LilypondHighlighter, self).__init__( parent )

        # tipos de palabras resaltadas: comandos, comentarios, notas,
        # comillas, símbolos raros de lilypond

        # comandos
        commandFormat = QtGui.QTextCharFormat()
        commandFormat.setForeground(QtCore.Qt.blue)
        commandFormat.setFontWeight(QtGui.QFont.Bold)

        commandPattern = QtCore.QRegExp("\\\\[a-zA-Z!\(\)<>]+")

        ## Diccionario con los pares (patrón, formato) con los que se dará
        #  color a los diferentes tipos de comandos de Lilypond
        self.highlightingRules = [ (commandPattern, commandFormat) ]

        # formato de las notas
        notesFormat = QtGui.QTextCharFormat()
        notesFormat.setFontWeight(QtGui.QFont.Bold)

        notesPattern = QtCore.QRegExp("\\b[a-gr](es|is){0,2}[,']?\\d*\\b")

        self.highlightingRules.append( ( notesPattern, notesFormat) )

        # formato del texto
        quotesFormat = QtGui.QTextCharFormat()
        quotesFormat.setForeground(QtCore.Qt.red)

        quotesPattern = QtCore.QRegExp("\".*\"")

        self.highlightingRules.append( ( quotesPattern, quotesFormat ) )

        # Símbolos malignos
        symbolFormat = QtGui.QTextCharFormat()
        symbolFormat.setForeground(QtCore.Qt.darkBlue)

        symbolPattern = QtCore.QRegExp("[\{\}',\^\[\]!?\-\./:=<>\(\)_]")

        self.highlightingRules.append( (symbolPattern, symbolFormat) )

        # Expresiones Scheme D:
        schemeFormat = QtGui.QTextCharFormat()
        schemeFormat.setForeground(QtCore.Qt.darkMagenta)

        schemePattern = QtCore.QRegExp("#.*[^\n]*")

        self.highlightingRules.append( (schemePattern, schemeFormat) )

        schemeFunctionOpen = QtCore.QRegExp("#\{")
        schemeFunctionClose = QtCore.QRegExp("#\}")

        #comentarios o_O... una de las partes más complicadas
        ## Formato de los comentarios de Lilypond
        self.commentFormat = QtGui.QTextCharFormat()
        self.commentFormat.setForeground(QtCore.Qt.darkGreen)

        singlelineCommentPattern = QtCore.QRegExp( "%.*[^\n]*" )

        self.highlightingRules.append( (singlelineCommentPattern, self.commentFormat) )

        #comentarios de bloque
        ## Expresión de apertura de bloque de comentario
        self.multilineCommentOpen = QtCore.QRegExp("%\{")
        ## Expresión de cierre de bloque de comentario
        self.multilineCommentClose = QtCore.QRegExp("%\}")

    ## Método que resalta los bloques de texto. Implementación del método abstracto
    #  de la clase madre QSyntaxHighlighter
    #  @param text El texto a resaltar
    def highlightBlock(self, text):
        for pattern, format in self.highlightingRules:
            expression = QtCore.QRegExp( pattern )
            index = expression.indexIn( text )
            while index >= 0:
                length = expression.matchedLength()
                self.setFormat( index, length, format )
                index = expression.indexIn( text, index + length )

        self.setCurrentBlockState(0)
        # estado del bloque:
        #  0 sin problemas
        #  1 bloque de comentarios sin cerrar aún

        # viene ahora la condenación de los bloques de comentarios
        startIndex = 0
        if self.previousBlockState() != 1:
            startIndex = self.multilineCommentOpen.indexIn( text )

        while startIndex >= 0:
            endIndex = self.multilineCommentClose.indexIn( text )

            if endIndex == -1: # si no encuentra
                self.setCurrentBlockState(1)
                commentLength = len(text) - startIndex
            else:
                commentLength = endIndex - startIndex + self.multilineCommentClose.matchedLength()

            self.setFormat( startIndex, commentLength, self.commentFormat )
            startIndex = self.multilineCommentOpen.indexIn( text, startIndex + commentLength )
                                
            
