#! /usr/bin/env python
# -*- coding: utf-8 -*-
######################################################################
## @package mdiWindow.py
# Clases para hacer funcionar la interfaz gráfica con muchas ventanas
# para poder tener varios archivos funcionando al mismo tiempo
#
# @author Javier Salazar Loyola
# @version 0.2
#
######################################################################

# para python 2
import sip
sip.setapi('QVariant', 2)

from PyQt4 import QtCore, QtGui

from textEditor import *

## Ventana principal de la aplicación.
class mdiWindow(QtGui.QMainWindow):
    ## Constructor que la deja completamente funcional
    def __init__(self, filename = QtCore.QString('')):
        super(mdiWindow, self).__init__()

        ## Área de trabajo de la interfaz
        self.mdiArea = QtGui.QMdiArea()
        self.mdiArea.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.mdiArea.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.setCentralWidget(self.mdiArea)

        self.mdiArea.subWindowActivated.connect(self.updateMenus)

        ## QSignalMapper de la ventana. Conecta señales sin parámetros con métodos
        #  que los requieran y sean la acción a ejecutar para la señal.
        self.windowMapper = QtCore.QSignalMapper(self)
        self.windowMapper.mapped[QtGui.QWidget].connect(self.setActiveSubWindow)

        self.createActions()
        self.createMenus()
        self.createToolBars()
        self.createStatusBar()
        self.updateMenus()

        self.center()        
        self.readSettings()
        self.createDock()
        
        self.setWindowTitle("Toccata")
        self.setWindowIcon(QtGui.QIcon('images/Icono.png'))
        ## título de la ventana, más completo
        self.titulo = QtCore.QString('Toccata - %1')
        self.setUnifiedTitleAndToolBarOnMac(True)

        if not filename.isEmpty():
            self.loadFile(filename)
        

    ## Define los pasos a seguir al cerrar el programa
    def closeEvent(self, event):
        self.mdiArea.closeAllSubWindows()
        if self.activeChild():
            event.ignore()
        else:
            self.writeSettings()
            event.accept()
            
    ## Crea un nuevo documento para edición
    def newFile(self):
        child = self.createChild()
        child.newFile()
        child.show()

    ## Abre un archivo directamente
    #  @param filename Nombre del archivo que abrir
    def loadFile(self, filename):
        child = self.createChild()
        if child.loadFile(fileName):
            self.statusBar().showMessage("Archivo cargado", 5000)
            child.show()
        else:
            child.close()
        
    ## Abre un archivo, si existe, devuelve el foco hacia esa ventana, en caso
    #  contrario, lo abre en una nueva
    def open(self):
        fileName = QtGui.QFileDialog.getOpenFileName(self, 'Abrir...',
                                                     self.directorio.absolutePath(),
                                                     "Lylipond files (*.ly);;All Files (*)")
        # verifica que esté abierto el archivo
        if fileName:
            fileInfo = QtCore.QFileInfo(fileName)
            self.directorio = fileInfo.dir()
                        
            existing = self.findChild(fileName)
            # si existe, lo centra
            if existing:
                self.mdiArea.setActiveSubWindow(existing)
                return

            # si no, crea la ventana y carga el archivo
            child = self.createChild()
            if child.loadFile(fileName):
                self.statusBar().showMessage("Archivo cargado", 5000)
                child.show()
            else:
                child.close()

    ## Salva el archivo actual
    #  @param child El editor que requiere guardar (implementado así por razones de diseño)
    #  @return True si guarda el archivo o no hay editores abiertos
    def save(self, child = None ):
        if not child:
            child = self.activeChild()

        self.setActiveChild( child )
        
        if child and child.isUntitled:
            if self.saveAs():
                self.statusBar().showMessage("Archivo guardado", 5000)
                return True
        elif child and child.save():
            self.statusBar().showMessage("Archivo guardado", 5000)
            return True
        else:
            return False
        return True
           

    ## Pregunta por nombre de archivo antes de guardar
    def saveAs(self):
        child = self.activeChild()

        if child:
            #and self.activeChild().saveAs( self.directorio.absolutePath() ):
            fileName = QtGui.QFileDialog.getSaveFileName(self, "Guardar como...",
                                                         self.directorio.absolutePath(),
                                                         "Lilypond Files (*.ly)" )
            if not fileName:
                return False
            if len(fileName) < 3 or fileName[-3:] != '.ly':#anexa el tipo de archivo por defecto
                fileName = fileName + '.ly'
            file = QtCore.QFile(fileName)

            fileInfo = QtCore.QFileInfo(file)#salva el directorio actual
            self.directorio = fileInfo.dir()
                    
            #guarda el archivo
            if child.saveFile(file):
                self.statusBar().showMessage("Archivo guardado", 5000)
                return True
            else:
                return False
            
    ## Cortar texto 
    def cut(self):
        if self.activeChild():
            self.activeChild().cut()
            
    ## Copiar texto
    def copy(self):
        if self.activeChild():
            self.activeChild().copy()

    ## Pegar Texto
    def paste(self):
        if self.activeChild():
            self.activeChild().paste()

    ## Información acerca del programa
    def about(self):
        QtGui.QMessageBox.about(self, 'Toccata',
                                'Segunda versi&oacute;n de <b>Toccata</b>, GUI para editar partituras<br>Esta versi&oacute;n es una suerte de IDE para Lilypond<br>Por Javier Salazar Loyola')

    ## Actualiza el estado de los menús (habilita o deshabilita según si hay editor abierto o no)
    def updateMenus(self):
        hasChild = (self.activeChild() is not None)
        self.saveAct.setEnabled(hasChild)
        self.saveAsAct.setEnabled(hasChild)
        self.pasteAct.setEnabled(hasChild)
        self.closeAct.setEnabled(hasChild)
        self.closeAllAct.setEnabled(hasChild)
        self.tileAct.setEnabled(hasChild)
        self.cascadeAct.setEnabled(hasChild)
        self.nextAct.setEnabled(hasChild)
        self.previousAct.setEnabled(hasChild)
        self.separatorAct.setVisible(hasChild)
        self.lilypond.setEnabled(hasChild)
        self.pdfReader.setEnabled(hasChild)
        
        hasSelection = (self.activeChild() is not None and
                        self.activeChild().textCursor().hasSelection())
        self.cutAct.setEnabled(hasSelection)
        self.copyAct.setEnabled(hasSelection)

    ## Agrega los menús a la ventana
    def updateWindowMenu(self):
        self.windowMenu.clear()
        self.windowMenu.addAction(self.closeAct)
        self.windowMenu.addAction(self.closeAllAct)
        self.windowMenu.addSeparator()
        self.windowMenu.addAction(self.tileAct)
        self.windowMenu.addAction(self.cascadeAct)
        self.windowMenu.addSeparator()
        self.windowMenu.addAction(self.nextAct)
        self.windowMenu.addAction(self.previousAct)
        self.windowMenu.addAction(self.separatorAct)
        windows = self.mdiArea.subWindowList()
        self.separatorAct.setVisible(len(windows) != 0)
        
        for i, window in enumerate(windows):
            child = window.widget()
            
            text = "%d %s" % (i + 1, child.userFriendlyCurrentFile())
            if i < 9:
                text = '&' + text
                
            action = self.windowMenu.addAction(text)
            action.setCheckable(True)
            action.setChecked(child == self.activeChild())
            action.triggered.connect(self.windowMapper.map)
            self.windowMapper.setMapping(action, child)
        
    ## Crea un editor de texto para el área de trabajo
    #  @return El editor recién creado
    def createChild(self):
        child = textEditor( self )
        self.mdiArea.addSubWindow(child)
        
        child.copyAvailable.connect(self.cutAct.setEnabled)
        child.copyAvailable.connect(self.copyAct.setEnabled)

        #self.connect(self.lilypond, QtCore.SIGNAL('triggered()'), child.compile )
        #self.connect(self.pdfReader, QtCore.SIGNAL('triggered()'), child.seePDF )
        
        return child

    ## Crea las acciones que manejan al progrma
    def createActions(self):
        ## Acción nuevo archivo
        self.newAct = QtGui.QAction(QtGui.QIcon('./images/new.png'), "&Nuevo",
                                    self, shortcut=QtGui.QKeySequence.New,
                                    statusTip="Crea un archivo en blanco", triggered=self.newFile)

        ## Acción abrir archivo
        self.openAct = QtGui.QAction(QtGui.QIcon('./images/open.png'),
                                     "&Abrir...", self, shortcut=QtGui.QKeySequence.Open,
                                     statusTip="Abre un archivo existente", triggered=self.open)

        ## Acción Guardar archivo
        self.saveAct = QtGui.QAction(QtGui.QIcon('./images/save.png'),
                                     "&Guardar", self, shortcut=QtGui.QKeySequence.Save,
                                     statusTip="Salva el documento", triggered=self.save)

        ## Acción Guardar como
        self.saveAsAct = QtGui.QAction("Guardar &como...", self,
                                       shortcut=QtGui.QKeySequence.SaveAs,
                                       statusTip="Guarda el archivo con un nuevo nombre",
                                       triggered=self.saveAs)

        ## Acción Salir
        self.exitAct = QtGui.QAction(QtGui.QIcon('./images/salir.png'), "S&alir", self, shortcut="Ctrl+Q",
                                     statusTip=u"Termina la aplicación",
                                     triggered=QtGui.qApp.closeAllWindows)

        ## Acción Cortar
        self.cutAct = QtGui.QAction(QtGui.QIcon('./images/cut.png'), "Cor&tar",
                                    self, shortcut=QtGui.QKeySequence.Cut,
                                    statusTip="Corta la selecci&oacute;n y la deja en el portapapeles",
                                    triggered=self.cut)

        ## Acción Copiar
        self.copyAct = QtGui.QAction(QtGui.QIcon('./images/copy.png'),
                                     "&Copiar", self, shortcut=QtGui.QKeySequence.Copy,
                                     statusTip=u"Copia la selección en el portapapeles",
                                     triggered=self.copy)

        ## Acción Pegar
        self.pasteAct = QtGui.QAction(QtGui.QIcon('./images/paste.png'),
                                      "&Pegar", self, shortcut=QtGui.QKeySequence.Paste,
                                      statusTip="Pega el contenido del portapapeles en el documento",
                                      triggered=self.paste)

        ## Acción Cerrar
        self.closeAct = QtGui.QAction("Ce&rrar", self, shortcut="Ctrl+W",
                                      statusTip="Cierra la ventana activa",
                                      triggered=self.mdiArea.closeActiveSubWindow)

        ## Acción Cerrar todo
        self.closeAllAct = QtGui.QAction("Cerrar &todo", self,
                                         statusTip="Cierra todas las ventanas",
                                         triggered=self.mdiArea.closeAllSubWindows)

        ## Acción agrupar ventanas en mosaico
        self.tileAct = QtGui.QAction("&Mosaico", self,
                                     statusTip="Muestra las ventanas en mosaico",
                                     triggered=self.mdiArea.tileSubWindows)

        ## Acción Cascada
        self.cascadeAct = QtGui.QAction("&Cascada", self,
                                        statusTip="Muestra las ventanas en cascada",
                                        triggered=self.mdiArea.cascadeSubWindows)

        ## Acción Siguiente ventana
        self.nextAct = QtGui.QAction("&Siguiente", self,
                                     shortcut=QtGui.QKeySequence.NextChild,
                                     statusTip="Mueve el foco a la ventana siguiente",
                                     triggered=self.mdiArea.activateNextSubWindow)

        ## Acción Ventana anterior
        self.previousAct = QtGui.QAction("Pre&via", self,
                                         shortcut=QtGui.QKeySequence.PreviousChild,
                                         statusTip="Cambia el foco a la ventana anterior",
                                         triggered=self.mdiArea.activatePreviousSubWindow)

        ## Acción que activa o desactiva el separador entre los comandos para navegar entre ventanas
        #  y la lista de ventanas
        self.separatorAct = QtGui.QAction(self)
        self.separatorAct.setSeparator(True)

        # Herramientas
        ## Acción compilar con Lilypond
        self.lilypond = QtGui.QAction(QtGui.QIcon("./images/icono2.png"), "&Lilypond", self,
                                      shortcut = "F1",
                                      statusTip = u"Genera la partitura para el archivo",
                                      triggered = self.compile)

        ## Acción ver PDF
        self.pdfReader = QtGui.QAction(QtGui.QIcon("./images/pdf.png"), "Ver &PDF", self,
                                       shortcut = "F2",
                                       statusTip = u"Muestra el PDF generado",
                                       triggered = self.readPDF)

        ## Acción Acerca de
        self.aboutAct = QtGui.QAction("&Acerca de...", self,
                                      statusTip=u"Información acerca del programa",
                                      triggered=self.about)

        ## Acción Acerca de Qt (con su propia ventanita integrada)
        self.aboutQtAct = QtGui.QAction("Acerca de &Qt", self,
                                        statusTip=u"Muestra el cuadro de la librería Qt",
                                        triggered=QtGui.qApp.aboutQt)
        

    ## Crea los menus y los organiza
    def createMenus(self):
        ## Menú archivo
        self.fileMenu = self.menuBar().addMenu("&Archivo")
        self.fileMenu.addAction(self.newAct)
        self.fileMenu.addAction(self.openAct)
        self.fileMenu.addAction(self.saveAct)
        self.fileMenu.addAction(self.saveAsAct)
        self.fileMenu.addSeparator()
        self.fileMenu.addAction(self.exitAct)

        ## Menú Edicion
        self.editMenu = self.menuBar().addMenu(u"&Edición")
        self.editMenu.addAction(self.cutAct)
        self.editMenu.addAction(self.copyAct)
        self.editMenu.addAction(self.pasteAct)

        # Herramientas van aquí Dx
        ## Menú Herramientas
        self.toolsMenu = self.menuBar().addMenu(u"Herramientas")
        self.toolsMenu.addAction(self.lilypond)
        self.toolsMenu.addAction(self.pdfReader)

        ## Menú Ventana
        self.windowMenu = self.menuBar().addMenu("&Ventana")
        self.updateWindowMenu()
        self.windowMenu.aboutToShow.connect(self.updateWindowMenu)

        self.menuBar().addSeparator()

        ## Menú Ayuda
        self.helpMenu = self.menuBar().addMenu("A&yuda")
        self.helpMenu.addAction(self.aboutAct)
        self.helpMenu.addAction(self.aboutQtAct)

    ## Crea la barra de herramientas para el programa
    def createToolBars(self):
        ## Barra de herramientas de Archivo
        self.fileToolBar = self.addToolBar("Archivo")
        self.fileToolBar.addAction(self.newAct)
        self.fileToolBar.addAction(self.openAct)
        self.fileToolBar.addAction(self.saveAct)

        ## Barra de herramientas de Edición
        self.editToolBar = self.addToolBar(u"Edición")
        self.editToolBar.addAction(self.cutAct)
        self.editToolBar.addAction(self.copyAct)
        self.editToolBar.addAction(self.pasteAct)

        ## Barra de herramientas de Herramientas
        self.toolsToolBar = self.addToolBar(u"Herramientas")
        self.toolsToolBar.addAction(self.lilypond)
        self.toolsToolBar.addAction(self.pdfReader)


    ## Crea la barra de estado
    def createStatusBar(self):
        self.statusBar().showMessage("Listo")

    ## Lee las configuraciones
    def readSettings(self):
        settings = QtCore.QSettings("The O.C.I.O. Company Ltd","Toccata")
        size = settings.value("mainWindow/size",
                              QtCore.QSize(640, 480) )
        isMax = settings.value("mainWindow/isMaximized",
                               False ) 
        
        ##Directorio de trabajo. El directorio que se abrirá por defecto con los cuadros
        # de diálogo y el directorio de trabajo de los procesos hijos
        self.directorio = QtCore.QDir(settings.value("Program/currentDir", "."))
        
        if isMax == "false":
            self.resize(size)
        else:
            self.showMaximized()


    ## Escribe las configuraciones
    def writeSettings(self):
        settings = QtCore.QSettings("The O.C.I.O. Company Ltd", "Toccata")
        if not self.isMaximized():
            settings.setValue( "mainWindow/size", self.size() )
            settings.setValue( "mainWindow/isMaximized", False )
        else:
            settings.setValue( "mainWindow/isMaximized", self.isMaximized() )

        settings.setValue( "Program/currentDir", self.directorio.absolutePath() )
            
        
    ## Averigua cuál es la ventana activa
    #  @return la ventana activa
    def activeChild(self):
        activeSubWindow = self.mdiArea.activeSubWindow()
        if activeSubWindow:
            return activeSubWindow.widget()
        return None

    ## Coloca el parámetro como la ventana activa
    #  @param child La venta a enfocar
    def setActiveChild(self, child):
        if child:
            for window in self.mdiArea.subWindowList():
                if window.widget().currentFile() == child.currentFile():
                    self.mdiArea.setActiveSubWindow( window )
                    return window
                return None
    

    ## Encuentra la ventana con el archivo abierto, si es que ya lo está.
    #  @param fileName El nombre de archivo que se quiera averiguar si está abierto
    #  @return La ventana que contiene al archivo solicitado
    def findChild(self, fileName):
        canonicalFilePath = QtCore.QFileInfo(fileName).canonicalFilePath()
        
        for window in self.mdiArea.subWindowList():
            if window.widget().currentFile() == canonicalFilePath:
                return window
            return None

    ## Hace que la ventana tenga el foco de atención
    #  @param window La ventana a focalizar
    def setActiveSubWindow(self, window):
        if window:
            #self.mdiArea.setActiveSubWindow(window)
            self.setActiveChild(window)

    ## Centra la ventana
    def center(self):
        screen = QtGui.QDesktopWidget().screenGeometry()
        size = self.geometry()
        self.move( (screen.width() - size.width())/2,
                   (screen.height() - size.height())/2)

    ## Liga la ventana activa con su compilación
    def compile(self):
        self.activeChild().compile(self.LogView)

    ## Liga la ventana activa con el PDF creado
    def readPDF(self):
        self.activeChild().seePDF()

    ##Método que crea una subventana que muestra el registro de salida de la compilación
    def createDock(self):
        ##Subventana que contiene el visor de registro de salida
        self.dock = QtGui.QDockWidget("Registro de salida", self)
        self.dock.setAllowedAreas(QtCore.Qt.BottomDockWidgetArea | QtCore.Qt.TopDockWidgetArea)
        
        self.dock.setFeatures(QtGui.QDockWidget.DockWidgetClosable or
                              QtGui.QDockWidget.DockWidgetMovable)
        
        ##El registro de salida
        self.LogView = QtGui.QTextEdit(self)
        self.dock.setWidget(self.LogView)
        self.addDockWidget(QtCore.Qt.BottomDockWidgetArea, self.dock)
                

# Corre el programa desde este archivo.
if __name__ == '__main__':
    import sys
    
    app = QtGui.QApplication(sys.argv)
    # chequea que se haya llamado con argumentos
    if len(sys.argv) > 1:
        mainwindow = mdiWindow(fileName = QtCore.QString(sys.argv[1]))
    else:
        mainwindow = mdiWindow()
    mainwindow.show()
    sys.exit(app.exec_())

