#! /usr/bin/env python
# -*- coding: utf-8 -*-
######################################################################
## @package textEditor.py
# Clase que contiene el workspace del mdi. Es el editor de texto
# principal, y, en el fondo, quien hace todos los llamados.
#
# @author Javier Salazar Loyola
# @version 0.2
#
######################################################################

from PyQt4 import QtGui, QtCore

from Highlighter import *
from Process import *

## Clase que implementa el editor de texto que contienen las ventanas
#  del área de trabajo de la interfaz MDI. Preparada para el trabajo
#  con las demás clases del programa.
class textEditor(QtGui.QTextEdit):
    ## Número de documento
    sequence = 1

    ## Constructor, sólo lo prepara para su existencia.
    def __init__(self, parent):
        super(textEditor, self).__init__()

        self.setAttribute(QtCore.Qt.WA_DeleteOnClose)
        ## Booleano que muestra si el documento tiene nombre o no.
        self.isUntitled = True
        ## Ventana que contiene al editor
        self.parent = parent

        # configuración de la fuente
        font = QtGui.QFont()
        font.setFamily('Courier')
        font.setFixedPitch(True)
        font.setPointSize(10)

        self.setTabStopWidth(12)
        self.setFont( font )
                                        

    ## Crea un nuevo archivo
    def newFile(self):
        self.isUntitled = True
        ## Nombre actual del archivo abierto con este editor
        self.curFile = "Documento nuevo %d" % textEditor.sequence
        textEditor.sequence += 1 # debe aumentar la secuencia

        self.setWindowTitle(self.curFile + "[*]") # título
        ## Resaltador del lenguaje
        self.highlighter = LilypondHighlighter(self.document()) # crea el resaltador

        # conecta con la modificación de contenidos
        self.document().contentsChanged.connect(self.documentWasModified)

    ## Carga un archivo previamente guardado.
    # @param filename El nombre del archivo.
    # @return False si es que no ha podido abrir el archivo
    def loadFile(self, filename):
        file = QtCore.QFile(filename)
        if not file.open(QtCore.QFile.ReadWrite | QtCore.QFile.Text):
            QtGui.QMessageBox.warning(self, "Toccata",
                                      "No se puede abrir el archivo %s:\n%s." % (fileName, file.errorString()))
            return False

        # formatea para colocar el texto
        inputStream = QtCore.QTextStream(file)
        QtGui.QApplication.setOverrideCursor(QtCore.Qt.WaitCursor)
        self.setPlainText(inputStream.readAll())
        self.highlighter = LilypondHighlighter(self.document())
        QtGui.QApplication.restoreOverrideCursor()

        self.setCurrentFile(filename)

        # conexión
        self.document().contentsChanged.connect(self.documentWasModified)

        return True

    ## Función provista por conveniencia. Abre el archivo en el que se guardará el texto.
    #  El guardado final lo realiza saveFile()
    def save(self):
        file = QtCore.QFile( self.curFile )
        print file.fileName()
        self.saveFile(file)
        

    ## La función que realiza el guardado final. Las demás se basan en esta para realizar
    #  el guardado del archivo.
    #  @param file El archivo donde guardar
    #  @return True si ha guardado correctamente
    def saveFile(self, file):
        if not file.open(QtCore.QFile.WriteOnly | QtCore.QFile.Text):
            QtGui.QMessageBox.warning(self, "Toccata",
                                      "No puede guardarse el archivo %s:\n%s." % (fileName, file.errorString()))
            return False
        
        outputStream = QtCore.QTextStream(file)
        QtGui.QApplication.setOverrideCursor(QtCore.Qt.WaitCursor)
        outputStream << self.toPlainText()
        QtGui.QApplication.restoreOverrideCursor()
        
        self.setCurrentFile(file.fileName())
        print "Guardado"
        return True

    ## Obtiene el nombre de archivo de forma más amigable
    #  @return el nombre del archivo
    def userFriendlyCurrentFile(self):
        return self.strippedName(self.curFile)

    ## Obtiene el nombre completo del archivo
    #  @return el nombre de archiv
    def currentFile(self):
        return self.curFile

    ## Verifica si cierra o no la ventana
    def closeEvent(self, event):
        if self.maybeSave():
            event.accept()
        else:
            event.ignore()

    ## Define que el documento ha sido modificado
    def documentWasModified(self):
        self.setWindowModified(self.document().isModified())

    ## Definie si es que ha de guardarse el archivo modificado en caso de salida
    #  @return False en caso de no querer cerrar ni guardar el programa, o lo que devuelva
    #          la función de guardar el archivo
    def maybeSave(self):
        if self.document().isModified():
            ret = QtGui.QMessageBox.warning(self, "Toccata",
                                            "'%s' ha sido modificado.\n&iquest;Desea guardar los cambios?"
                                            % self.userFriendlyCurrentFile(),
                                            QtGui.QMessageBox.Save | QtGui.QMessageBox.Discard |
                                            QtGui.QMessageBox.Cancel)
            if ret == QtGui.QMessageBox.Save:
                print "Guardando archivo..."
                return self.parent.save( self )
            elif ret == QtGui.QMessageBox.Cancel:
                return False
            
        return True

    ## Configura parámetros del archivo abierto
    #  @param fileName el nombre del archivo
    def setCurrentFile(self, fileName):
        self.curFile = QtCore.QFileInfo(fileName).canonicalFilePath()
        self.isUntitled = False
        self.document().setModified(False)
        self.setWindowModified(False)
        self.setWindowTitle(self.userFriendlyCurrentFile() + "[*]")

    ## Obtiene el nombre de archivo sin el recorrido de directorios
    #  @return el nombre del archivo
    def strippedName(self, fullFileName):
        return QtCore.QFileInfo(fullFileName).fileName()

    ## Invoca a Lilypond y hace la conversión a PDF
    #  @param logWidget QTextEdit donde se visualizarán los resultados
    #  @return False si no se pudo realizar la acción
    def compile(self, logWidget):
        if not self.maybeSave():
            QtGui.QMessageBox.information(self.parent,'Advertencia',
                                          "No se pudo guardar el archivo, no se podrá compilar",
                                          QtGui.QMessageBox.Ok)
            return False

        compiler = LilypondProcess( self, logWidget )
        compiler.start()

        return True

    ## Inicia el visor de PDF
    def seePDF(self):
        reader = ReaderProcess( self )

        reader.start()

        return True
    
        
